//
//  TRMovieInspector.m
//  Tryst
//
//  Created by Francisco Tolmasky on Thu Nov 13 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import "TRMovieImageView.h"
#import "TRMovieFileView.h"
#import "TRMovieGutter.h"
#import "TRMovieScrollView.h"
#import "TRMovieInspector.h"


@interface TRMovieScrollView (friend)
- (void)synchronize;
- (void)showGutter:(id)sender;
- (void)hideGutter:(id)sender;
- (void)updateGutterHeight;
- (void)evaluateNecessityOfVerticalScroller;
@end

@interface TRMovieInspector (drawing)
- (void)drawRaisedEditorIfNecessaryInRect:(NSRect)clipRect;
- (void)drawRolloverIfNecessaryInRect:(NSRect)clipRect;
- (void)drawImageIfNecessaryInRect:(NSRect)clipRect;
@end

@implementation TRMovieInspector

- (id)initWithCoder:(NSCoder *)aCoder
{
    self= [super initWithCoder: aCoder];
    if(self)
    {
        currentEditedStartLocation= NSNotFound;
        currentEditedYearLocation= NSNotFound;
        
        rollover.lhsRange= kNotFoundRange;
        rollover.rhsRange= kNotFoundRange;
        
        temporaryIdentifierCount= 0;
        
        [self setRichText: NO];
		//[self setImportsGraphics: NO];
    }
    return self;
}

- (id)initWithFrame: (NSRect)aFrame
{
    self= [super initWithFrame: aFrame];
    if(self)
    {
        currentEditedStartLocation= NSNotFound;
        currentEditedYearLocation= NSNotFound;
        
        rollover.lhsRange= kNotFoundRange;
        rollover.rhsRange= kNotFoundRange;
        
        temporaryIdentifierCount= 0;
        
        [self setRichText: NO];
		//[self setImportsGraphics: NO];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver: self];
    
    [model release];
    
	[coverArt release];
	
	[rollover.image release];
    
    [super dealloc];
}

- (NSArray *)modelOptions
{
	return nil;
}

- (void)setModelOptions:(NSArray *)anArray
{
	[self setModel: [anArray count]?[anArray objectAtIndex: 0]:nil];
}

- (void)setModel:(TRMutableMovie *)aModel
{
    if(model==aModel) return;
    
    if([self isEditable]) [self recordValuesAndProperties];
    
    if(model && !aModel) [coverArt removeFromSuperview];
    else if(!model && aModel) [self addSubview: coverArt];

    [model release];
    model= [aModel retain];
    
    [self loadProperties];
    [self setNeedsDisplay: YES];
    
    if([self isEditable]) [self setSelectedRange: [self nextEditableRange: 0 forward: YES]];
    
    [self discardCursorRects];
    [self resetCursorRects];
    
    if(!model && [self isEditable]) [self setEditable: NO]; 
}

- (TRMovie *)model
{
    return model;
}

- (void)setMovieGutter:(TRMovieGutter *)aMovieGutter
{
    gutter= aMovieGutter;
    [gutter setMovieInspector: self];
}

- (TRMovieGutter *)movieGutter
{
    return gutter;
}

- (void)awakeFromNib
{
    // Set up Cover Art
    coverArt= [[TRMovieImageView alloc] initWithFrame: NSMakeRect(kLeftTabLocation-48.0,14.0,48.0,48.0) movieInspector: self];
    [coverArt bind: @"data" toObject: self withKeyPath: @"model.imageData" options:
        [NSDictionary dictionaryWithObject: [[NSImage imageNamed: @"Clacker"] TIFFRepresentation] forKey: @"NSNullPlaceholder"]];
    [coverArt bind: @"editable" toObject: self withKeyPath: @"editable" options: nil];
    
    // Set up Property Viewer
    /*NSTextStorage *textStorage= [self textStorage];
    NSAttributedString *tab= [[[NSAttributedString alloc] initWithString: @"\t"] autorelease];
    NSAttributedString *newline= [[[NSAttributedString alloc] initWithString: @"\n"] autorelease];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @"Movie Title"
        attributes: [self placeholderAttributesForProperty: kTRMovieTitleProperty identifier: @"" placeholderString: @"Movie Title" font: [NSFont boldSystemFontOfSize: 14]]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @"  (" 
        attributes: [NSDictionary dictionaryWithObject: [NSFont boldSystemFontOfSize: 14] forKey: NSFontAttributeName]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @"Year" 
        attributes: [self placeholderAttributesForProperty: kTRMovieYearProperty identifier: @"" placeholderString: @"Year" font: [NSFont boldSystemFontOfSize: 14]]]];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @") "
        attributes: [NSDictionary dictionaryWithObject: [NSFont boldSystemFontOfSize: 14] forKey: NSFontAttributeName]]];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: tab];
    [textStorage appendAttributedString: [[NSAttributedString alloc] initWithString: @"Genre"
        attributes: [self placeholderAttributesForProperty: kTRMovieGenreProperty identifier: @"" placeholderString: @"Genre" font: [NSFont systemFontOfSize: 11]]]];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    [textStorage appendAttributedString: newline];
    
    [textStorage removeAttribute: NSParagraphStyleAttributeName range: NSMakeRange(0,[textStorage length])];
    [textStorage addAttributes: [self tabStopAttributesForScalarProperties] range: NSMakeRange(0,[textStorage length])];*/
    
    // We don't have to do this, but its convenient for demo purposes.
    [self setDelegate: self];
}

// Editing

- (void)toggleEditable:(id)sender
{
    [self setEditable: ![self isEditable]];
}

- (BOOL)editable
{
    return [self isEditable];
}

- (void)setEditable:(BOOL)allowsEditing
{
    if(!allowsEditing && [self isEditing]) [self endEditing];
    [super setEditable: allowsEditing];
    if(allowsEditing)
    {
        [(TRMovieScrollView *)[self enclosingScrollView] showGutter: self];
        [[self window] makeFirstResponder: self];
        [self loadProperties];
        [self setSelectedRange: [self nextEditableRange: 0 forward: YES]];
    }
    else
    {
        [(TRMovieScrollView *)[self enclosingScrollView] hideGutter: self];
        [self recordValuesAndProperties];
        [self loadProperties];
    }
}

- (void)resetCursorRects
{
    if(!model)
	{
		[self addCursorRect: [self visibleRect] cursor: [NSCursor arrowCursor]];
		return;
	}
	
    [super resetCursorRects];
    
    NSTextStorage *textStorage= [self textStorage];
    NSLayoutManager *layoutManager= [self layoutManager];
    NSTextContainer *textContainer= [self textContainer];
    
    NSRect  visibleRect= [self visibleRect],
            editingGlyphRect;
    NSRange effectiveRange;
    
    NSCursor *cursor;
    
    // Set up Properties Cursor Rects
    effectiveRange.location= [layoutManager characterIndexForGlyphAtIndex: [layoutManager glyphIndexForPoint: visibleRect.origin inTextContainer: textContainer]];

    while(effectiveRange.location<[[[self textStorage] string] length]-1)
    {
        cursor= [textStorage attribute: NSCursorAttributeName atIndex: effectiveRange.location effectiveRange: &effectiveRange];
        if(cursor && cursor!=[NSCursor IBeamCursor])
        {
            editingGlyphRect=[layoutManager
                boundingRectForGlyphRange: [layoutManager glyphRangeForCharacterRange: effectiveRange actualCharacterRange: NULL] 
                inTextContainer: textContainer];
            [self addCursorRect: NSIntersectionRect(visibleRect,editingGlyphRect) cursor: cursor];
        }
        effectiveRange.location= NSMaxRange(effectiveRange);
    }
    
    // Set up Cover Art Cursor Rect
    [self addCursorRect: NSIntersectionRect(visibleRect,[coverArt frame]) cursor: [NSCursor arrowCursor]];
}

- (void)setFrameSize:(NSSize)aSize
{
    [super setFrameSize: aSize];
    [(TRMovieScrollView *)[self enclosingScrollView] updateGutterHeight];
    [(TRMovieScrollView *)[self enclosingScrollView] evaluateNecessityOfVerticalScroller];
}

// Don't allow continuous spell checking
- (BOOL)isContinuousSpellCheckingEnabled
{
    return NO;
}

// --- Drawing Override ---

- (void)drawRect:(NSRect)clipRect
{
    if(!model)
    {
        static NSAttributedString * TRMovieInspectorNoSelectionString= nil;
        static NSSize               TRMovieInspectorNoSelectionSize;
        
        if(!TRMovieInspectorNoSelectionString)
        {
            TRMovieInspectorNoSelectionString= [[NSAttributedString alloc] initWithString: NSLocalizedString(@"No Selection",@"No Selection")
                attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont boldSystemFontOfSize: 14.0], [NSColor grayColor], nil] 
                forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]]];
            TRMovieInspectorNoSelectionSize= [TRMovieInspectorNoSelectionString size];
        }
        
        [[NSColor whiteColor] set];
        NSRectFill(clipRect);
        
        [TRMovieInspectorNoSelectionString drawAtPoint: NSMakePoint(([self frame].size.width-TRMovieInspectorNoSelectionSize.width)/2,([self frame].size.height-TRMovieInspectorNoSelectionSize.height)/2)];
        
        return;
    }
    
    // Let NSTextView do its thing man...
    [super drawRect: clipRect];
    
    // Clear white region
    [[NSColor whiteColor] set];
    NSRectFill(NSMakeRect(5.0,0.0,kLeftTabLocation,87.0));
    
    if([self isEditable])
        // Draw the raised editor
        [self drawRaisedEditorIfNecessaryInRect: clipRect];  
    else
        // Draw the rollover
        [self drawRolloverIfNecessaryInRect: clipRect];
}

@end