//
//  TRInlineTextControls.m
//  Tryst
//
//  Created by Francisco Tolmasky on Wed Jan 14 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "TRMovieInspector.h"
#import "TRMovieLabelCell.h"


@interface TRMovieInspector (friend)
- (void)updateLayoutForAttachment:(NSTextAttachment *)anAttachment;
- (void)runCustomLabelSheet:(id)sender;
@end

static NSMutableDictionary* TRMovieSharedLabels;
static NSMutableDictionary* TRMovieSynchronizedMenus;

@implementation TRMovieLabelCell

+ (void)initialize
{
    if([self class]==[TRMovieLabelCell class])
    {
        TRMovieSharedLabels= [[NSMutableDictionary alloc] init];
        TRMovieSynchronizedMenus= [[NSMutableDictionary alloc] init];
        
        [self setLabels: [NSArray arrayWithObjects: @"director", @"assistant director", @"producer", @"executive producer", nil] forProperty: kTRMovieCrewProperty];
        [self setLabels: [NSArray arrayWithObjects: @"actor", @"supporting actor", @"voice", nil] forProperty: kTRMovieCastProperty];
    }
}

+ (void)setLabels:(NSArray *)labels forProperty:(NSString *)aProperty
{
    [TRMovieSharedLabels setObject: labels forKey: aProperty];
}

+ (void)addLabel:(NSString *)aLabel forProperty:(NSString *)aProperty
{
    NSArray *synchronizedMenus= [TRMovieSynchronizedMenus objectForKey: aProperty];
    int i= 0,
        length= [synchronizedMenus count];
    for(;i<length;++i)
        [[synchronizedMenus objectAtIndex: i] insertItemWithTitle: aLabel atIndex: [[synchronizedMenus objectAtIndex: i] numberOfItems]-1];
}

- (id)initWithProperty:(NSString *)aProperty movieInspector:(TRMovieInspector *)aMovieInspector
{
    self= [super initTextCell: @"" pullsDown: NO];
    if(self)
    {
        movieInspector= aMovieInspector;
        representedProperty= [aProperty retain];
        
        [self setBordered: NO];
        [self setControlSize: NSMiniControlSize];
        [self setFont: [NSFont boldSystemFontOfSize: 11]];
        [self addItemsWithTitles: [TRMovieSharedLabels objectForKey: aProperty]];
        
        [[self menu] addItem: [NSMenuItem separatorItem]];
        [self addItemWithTitle: @"Custom..."];
        
        customMenuItem= [self itemAtIndex: [self numberOfItems]-1];
    }
    return self;
}

- (void)dealloc
{
	[representedProperty release];
	
	[super dealloc];
}

- (NSString *)representedProperty
{
    return representedProperty;
}

- (void)addCustomLabel:(NSString *)aCustomLabel select:(BOOL)select
{
    unsigned index= [self indexOfItemWithTitle: aCustomLabel];
    if(index==-1) [self insertItemWithTitle: aCustomLabel atIndex: index= [self numberOfItems]-2];
    if(select) [self selectItemAtIndex: index];
}

- (void)selectItemAtIndex:(int)anIndex
{
    if([self itemAtIndex: anIndex]==customMenuItem) [movieInspector runCustomLabelSheet: self];
    else
    {
        [super selectItemAtIndex: anIndex];
        if(attachment) [movieInspector updateLayoutForAttachment: attachment];
    }
}

- (NSTextAttachment *)attachment
{
    return attachment;
}

- (void)setAttachment:(NSTextAttachment *)anAttachment
{
    attachment= anAttachment;
}

- (NSPoint)cellBaselineOffset
{
    return NSMakePoint(0.0,-3.0);
}

- (NSRect)cellFrameForTextContainer:(NSTextContainer *)textContainer proposedLineFragment:(NSRect)lineFrag glyphPosition:(NSPoint)position characterIndex:(unsigned)charIndex
{
    return NSMakeRect([self cellBaselineOffset].x,[self cellBaselineOffset].y,[self cellSize].width,[self cellSize].height);
}

- (NSSize)cellSize
{
    NSSize size= [[[self selectedItem] title] sizeWithAttributes: [NSDictionary dictionaryWithObject: [self font] forKey: NSFontAttributeName]];
    return NSMakeSize(size.width+10.0>kLeftTabLocation-5.0?kLeftTabLocation-5.0:size.width+10.0,14.0);
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)aView
{
    static NSMutableParagraphStyle *truncatingParagraphStyle;
    if(!truncatingParagraphStyle)
    {
        truncatingParagraphStyle= [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
        [truncatingParagraphStyle setLineBreakMode: NSLineBreakByTruncatingTail];
    }
    
    [[[self selectedItem] title] drawInRect: NSMakeRect(cellFrame.origin.x,cellFrame.origin.y,cellFrame.size.width-10.0,cellFrame.size.height)
        withAttributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [self font], truncatingParagraphStyle, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSParagraphStyleAttributeName, nil]]];
    [[NSImage imageNamed: @"Inline Menu Arrows"]
        compositeToPoint: NSMakePoint(cellFrame.origin.x+[self cellSize].width-5.0,cellFrame.origin.y+[self cellSize].height-3.0)
        operation: NSCompositeSourceOver];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)aView characterIndex:(unsigned)charIndex
{
    [self drawWithFrame: cellFrame inView: aView];
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView characterIndex:(unsigned)charIndex layoutManager:(NSLayoutManager*)layoutManager
{
    [self drawWithFrame: cellFrame inView: controlView];
}

- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView
{
    [super highlight: flag withFrame: cellFrame inView: aView];
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)aTextView untilMouseUp:(BOOL)flag
{
    return [super trackMouse: theEvent inRect: cellFrame ofView: aTextView untilMouseUp: flag];
}

- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)aTextView atCharacterIndex:(unsigned)charIndex untilMouseUp:(BOOL)flag
{
    return [self trackMouse: theEvent inRect: cellFrame ofView: aTextView untilMouseUp: flag];
}

- (BOOL)wantsToTrackMouse
{
    return YES;
}

- (BOOL)wantsToTrackMouseForEvent:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView atCharacterIndex:(unsigned)charIndex
{
    return YES;
}

@end
