//
//  TRWindowController.m
//  Tryst
//
//  Created by Francisco Tolmasky on Tue Nov 18 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

//#import "TRBugReport.h"

#import "TRBroadcasting.h"

#import "TRMovieFileView.h"
#import "TRMovieTableView.h"
#import "TRMovieLabelCell.h"
#import "TRMovieInspector.h"
#import "TRMovieScrollView.h"
#import "TRIntegerControl.h"
#import "TRTimeIntervalControl.h"

#import "TRController.h"
#import "TRWindowController.h"
#import "TRMovieListingController.h"


static NSString *   TRWindowFrameAutosaveName=      @"Window Frame Autosave Name";
static NSString *   TRRecentsAutosaveName=          @"Recents Autosave Name";

static NSString *   TRWindowDisplayModeKey=			@"Window Display Mode";
static NSString *   TRWindowCollapsedKey=           @"Window Collapsed";
static NSString *   TRMovieSearchKey=               @"Movie Search";
static NSString *   TRMovieListWidthKey=            @"Movie List Width";
static NSString *   TRAuxiliaryViewWidthKey=        @"Auxiliary View Width";

static NSString *   TRBroadcastTimeRemainingKey=	@"Broadcast Time Remaining";
static NSString *   TRBroadcastConnectionLimitKey=  @"Broadcast Connection Limit";
static NSString *   TRBroadcastRequirePasswordKey=  @"Broadcast Requires Password";
static NSString *   TRBroadcastPasswordKey=			@"Broadcast Password";

@implementation TRWindowController

+ (void)initialize
{
    if(self==[TRWindowController class])
    {
        NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
        NSDictionary *registeredDefaults=
        [[NSDictionary alloc]
            initWithObjects: [NSArray arrayWithObjects: 
				[NSNumber numberWithInt: TRWindowLibraryDisplayMode],
                [NSNumber numberWithBool: NO], 
                [NSNumber numberWithFloat: 200.0],
                [NSNumber numberWithFloat: 394.0],
                kTRMovieTitleProperty,
				[NSNumber numberWithDouble: 300],
				[NSNumber numberWithInt: 5],
				[NSNumber numberWithBool: NO],
				@"", nil]
            forKeys: [NSArray arrayWithObjects: 
				TRWindowDisplayModeKey,
                TRWindowCollapsedKey,
                TRMovieListWidthKey,
                TRAuxiliaryViewWidthKey, 
                TRMovieSearchKey,
				TRBroadcastTimeRemainingKey,
				TRBroadcastConnectionLimitKey,
				TRBroadcastRequirePasswordKey,
				TRBroadcastPasswordKey, nil]];
        [defaults registerDefaults: registeredDefaults];
        [registeredDefaults release];
    }
}

- (id)initWithController:(TRController *)aController
{
    self= [super initWithWindowNibName: @"TRWindow"];
    if(self)
    {
        controller= aController;
        
        isCollapsed= [[[NSUserDefaults standardUserDefaults] objectForKey: TRWindowCollapsedKey] boolValue];
        collapseInProgress= NO;
        movieListRowHeight= 52.0;
    }
    return self;
}

- (void)dealloc
{
	[movieInspector release];
    [super dealloc];
}

- (void)windowDidLoad
{
    NSWindow *window= [self window];
    NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
	NSNotificationCenter *defaultNotificationCenter= [NSNotificationCenter defaultCenter];
    
    // Return Window to previous state.
	[window setMinSize: NSMakeSize(isCollapsed?194.0:620.0,450.0)];
    [self setWindowFrameAutosaveName: TRWindowFrameAutosaveName];
    [window setFrameFromString: [self windowFrameAutosaveName]];
	[self setDisplayMode: (TRWindowDisplayMode)[[defaults objectForKey: TRWindowDisplayModeKey] intValue]];
    
    // Register self for size changing notification for left view
    [defaultNotificationCenter
        addObserver: self
        selector: @selector(adjustDependentViews:)
        name: NSViewFrameDidChangeNotification
        object: [[mainSplitView subviews] objectAtIndex: 0]];
	
    // Prepare Search Field
    NSMenu *template= [[searchField cell] searchMenuTemplate];
    
    [template setDelegate: template];
    [[template itemAtIndex: 0] setRepresentedObject: kTRMovieTitleProperty];
    [[template itemAtIndex: 1] setRepresentedObject: kTRMovieGenreProperty];
    
    [movieListingController changeSearchKey: [template itemAtIndex: [[defaults objectForKey: TRMovieSearchKey] isEqualTo: kTRMovieGenreProperty]]];
    
    [[searchField cell] setSearchMenuTemplate: nil];
    [[searchField cell] setSearchMenuTemplate: template];
    
    [searchField setRecentsAutosaveName: TRRecentsAutosaveName];
    [searchField recentSearches];  // Load recent searches if archived...
	
	[passwordTextField bind: @"enabled" toObject: self withKeyPath: @"requirePasswordCheckBox.state" options: nil];

	// Prepare toolbar buttons
    [toolbarContainer setHidden: isCollapsed];

	// Prepare Movie ScrollView
	[movieScrollView retain];
	
	// Prepare Movie Inspector
    [movieInspector setEditable: NO]; //Not Editable by default.
    [movieInspector bind: @"modelOptions" toObject: movieListingController withKeyPath: @"selectedObjects" options: nil]; //Bind to controller
	
    // Prepare Edit Button
    [editButton bind: @"enabled" toObject: inspectorController withKeyPath: @"content.model" options: [NSDictionary dictionaryWithObject: NSIsNotNilTransformerName forKey: @"NSValueTransformerName"]];
    [editButton bind: @"state" toObject: inspectorController withKeyPath: @"content.editable" options: nil];
	
    // Prepare Library Controller
	if(displayMode==TRWindowLibraryDisplayMode) [movieListingController bind: @"contentArray" toObject: controller withKeyPath: @"library" options: nil];
	//[movieListingController setContent: [controller library]];
	else [movieListingController bind: @"contentArray" toObject: [controller client] withKeyPath: @"movies" options: nil];
	//[movieListingController setContent: [[controller client] movies]];
	[self setDisplayMode: displayMode];
	
	[movieListingController setSearchField: searchField];
    [movieListingController setMovieInspector: movieInspector];
    
    // Prepare Main Split View
    [window setMinSize: NSMakeSize(isCollapsed?194.0:620.0,450.0)];
    [[[mainSplitView subviews] objectAtIndex: 0]
        setFrameSize: NSMakeSize([[defaults objectForKey: TRMovieListWidthKey] floatValue],
                                 [mainSplitView frame].size.height)];
	
	if(!isCollapsed)
    {
        [[[mainSplitView subviews] objectAtIndex: 1]
            setFrameSize: NSMakeSize([[defaults objectForKey: TRAuxiliaryViewWidthKey] floatValue],
                                     [mainSplitView frame].size.height)];
        [mainSplitView adjustSubviews];
    }
    else
        [[[mainSplitView subviews] objectAtIndex: 1]
            setFrameSize: NSMakeSize(0.0,[mainSplitView frame].size.height)];
    [mainSplitView setDelegate: self];
	
	// Prepare Broadcast Sheet
	int connectionLimit= [[defaults objectForKey: TRBroadcastConnectionLimitKey] intValue];
	
	[timeRemainingControl setTimeInterval: [[defaults objectForKey: TRBroadcastTimeRemainingKey] doubleValue]];
	[connectionLimitCheckBox setState: connectionLimit>0?NSOnState:NSOffState];
	[connectionLimitControl setIntValue: abs(connectionLimit)];
	[requirePasswordCheckBox setState: [[defaults objectForKey: TRBroadcastRequirePasswordKey] boolValue]];
	[passwordTextField setStringValue: [defaults objectForKey: TRBroadcastPasswordKey]];
	
	[self synchronizeBroadcastOptions: self];
	
	// Prepare Broadcasting Notifications
	
	[defaultNotificationCenter
        addObserver: self
        selector: @selector(reloadMovieData:)
        name: TRBroadcastServerInformationDidChangeNotification
        object: [controller server]];

	[defaultNotificationCenter
        addObserver: self
        selector: @selector(updateMovieListing:)
        name: TRBroadcastClientInformationDidChangeNotification
        object: [controller client]];
		
	NSLog([[NSHost currentHost] address]);
}

- (void)setDisplayMode:(TRWindowDisplayMode)aDisplayMode
{
	[displayModeTabView selectTabViewItemAtIndex: aDisplayMode==TRWindowLibraryDisplayMode?0:1];
//	[self tabView: displayModeTabVIew
}

- (TRWindowDisplayMode)displayMode
{
	return displayMode;
}

- (void)reloadMovieData:(NSNotification *)aNotification
{
	[movieListing reloadData];
}

- (void)updateMovieListing:(NSNotification *)aNotification
{
	[movieListingController rearrangeObjects];
}

- (void)setCollapsed:(BOOL)collapseFlag
{
    if(isCollapsed==collapseFlag) return;
	
	if(collapseFlag) [toolbarContainer setHidden: YES];
    
    NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
    NSRect windowFrame= [[self window] frame];
    [mainSplitView adjustSubviews];
    windowFrame.size.width+= ((isCollapsed= collapseFlag)?-1:1)*
        ([mainSplitView dividerThickness]+[[defaults objectForKey: TRAuxiliaryViewWidthKey] floatValue]);
    collapseInProgress= YES;
    [[self window] setFrame: windowFrame display: YES animate: YES];
    [[self window] setMinSize: NSMakeSize(isCollapsed?194.0:620.0,450.0)];
    collapseInProgress= NO;
    [defaults
        setObject: [NSNumber numberWithBool: isCollapsed]
        forKey: TRWindowCollapsedKey];
	
	if(!isCollapsed) [toolbarContainer setHidden: NO];
}

- (BOOL)collapsed
{
    return isCollapsed;
}

- (IBAction)toggleMovieInspector:(id)sender
{
    [self setCollapsed: !isCollapsed];
}

- (IBAction)synchronizeBroadcastOptions:(id)sender
{	
	[connectionLimitControl setEnabled: [connectionLimitCheckBox state]==NSOnState];
	[passwordTextField setEnabled: [requirePasswordCheckBox state]==NSOnState];
}

- (IBAction)runBroadcastSheet:(id)sender
{
	[NSApp beginSheet: broadcastSheet
		modalForWindow: [self window]
		modalDelegate: self
		didEndSelector: @selector(broadcastSheetDidEnd: returnCode: movie:)
		contextInfo: [[movieListingController selectedObjects] objectAtIndex: 0]];
}

- (IBAction)endBroadcastSheet:(id)sender
{
	[NSApp endSheet: broadcastSheet returnCode: [sender tag]];
	
	[broadcastSheet orderOut: self];
}

- (IBAction)handleBroadcast:(id)sender
{
	if([[sender title] isEqualTo: @"Broadcast"])
		[self runBroadcastSheet: self];
	else if([[sender title] isEqualTo: @"Cancel"])
	{
		[[controller server] stop];
		
		[broadcastButton setTitle: @"Broadcast"];
	}
}

- (void)broadcastSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode movie:(TRMutableMovie *)aMovie
{
	NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
	NSTimeInterval timeRemaining= [timeRemainingControl timeInterval];
	unsigned connectionLimit= [connectionLimitControl intValue];
	
	[defaults setObject: [NSNumber numberWithDouble: timeRemaining] forKey: TRBroadcastTimeRemainingKey];
	[defaults setObject: [NSNumber numberWithInt: [connectionLimitCheckBox state]==NSOnState?connectionLimit:-connectionLimit]
		forKey: TRBroadcastConnectionLimitKey];
	[defaults setObject: [NSNumber numberWithBool: [requirePasswordCheckBox state]==NSOnState] forKey: TRBroadcastRequirePasswordKey];
	[defaults setObject: [passwordTextField stringValue] forKey: TRBroadcastPasswordKey];
	
	if(returnCode)
	{
		[[controller server]
			broadcastMovie: aMovie
			timeRemaining: [timeRemainingControl timeInterval]
			connectionLimit: [connectionLimitCheckBox state]==NSOnState?connectionLimit:0
			password: [requirePasswordCheckBox state]==NSOnState?[passwordTextField stringValue]:nil];
			
		[broadcastButton setTitle: @"Cancel"];
	}
}

- (void)subscribeWithPassword:(NSString *)aPassword movie:(TRMovie *)aMovie
{
	TRBroadcastResponse response= [(TRBroadcastServer *)[aMovie valueForProperty: kTRBroadcastServerProperty]
		subscribeClient: [controller client] withPassword: aPassword];

	if(response<0)
		NSBeginAlertSheet(
			@"Subscription Request Rejected",
			@"OK", nil, nil,
			[self window],
			nil,
			NULL,//@selector(sheetDidEnd: returnCode: contextInfo:),
			NULL,
			NULL,
			response==TRBroadcastPasswordInvalidResponse?
			NSLocalizedString(@"The password entered to subscribe this movie was incorrect.","Incorrect Password"):
			NSLocalizedString(@"The connection limit for this movie has been reached.", @"Connection Limit Reached"));
	else [movieListing reloadData];
}

- (IBAction)subscribe:(id)sender
{
	if(displayMode==TRWindowBrowseDisplayMode)
	{
		NSArray *selectedObjects= [movieListingController selectedObjects];
		if([selectedObjects count]==1)
		{
			TRMovie *movie= [selectedObjects objectAtIndex: 0];
			TRBroadcastServer *server= [movie valueForProperty: kTRBroadcastServerProperty];
			if([server totalConnections]==[server connectionLimit])
			{
				NSBeginAlertSheet(
					@"Could Not Subscribe",
					@"OK", nil, nil,
					[self window],
					nil,
					NULL,//@selector(sheetDidEnd: returnCode: contextInfo:),
					NULL,
					NULL,
					NSLocalizedString(@"This movie's subscription slots are currently full.", @"Subscription Slots Full"));
			}
			else if([server isPasswordProtected])
			{
				[passwordEntryTextField setStringValue: @""];
				
				[NSApp beginSheet: passwordSheet
					modalForWindow: [self window]
					modalDelegate: self
					didEndSelector: @selector(passwordSheetDidEnd: returnCode: movie:)
					contextInfo: movie];
			}
			else [self subscribeWithPassword: nil movie: movie];
		}
	}
}

- (IBAction)endPasswordSheet:(id)sender
{
	[passwordSheet orderOut: self];
	
	[NSApp endSheet: passwordSheet returnCode: [[sender selectedCell] tag]];
}

- (void)passwordSheetDidEnd:(NSPanel *)sheet returnCode:(int)returnCode movie:(TRMovie *)movie
{
	if(returnCode) [self subscribeWithPassword: [passwordEntryTextField stringValue] movie: movie];
}

- (void)adjustDependentViews:(NSNotification *)aNotification
{
    [searchContainer setFrameSize: NSMakeSize([[aNotification object] frame].size.width-40.0,22.0)];
	[controlsContainer setFrameSize: NSMakeSize([[aNotification object] frame].size.width-12.0,18.0)];
    [editButton setFrameOrigin: NSMakePoint([[aNotification object] frame].size.width+[mainSplitView dividerThickness]+28.0,[editButton frame].origin.y)];
    [[searchContainer superview] setNeedsDisplay: YES];
    [[editButton superview] setNeedsDisplay: YES];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	if([movieInspector isEditable]) [movieInspector setEditable: NO];

    [[NSUserDefaults standardUserDefaults] setObject: [[[[searchField cell] searchMenuTemplate] itemAtIndex: 0] state]?kTRMovieTitleProperty:kTRMovieGenreProperty forKey: TRMovieSearchKey];
}

/*- (IBAction)reportBug:(id)sender
{
	[TRBugReport runBugReportForWindow: [self window]];
}*/

@end

@implementation TRWindowController (NSTabViewDelegate)

- (void)tabView:(NSTabView *)aTabView
	didSelectTabViewItem:(NSTabViewItem *)aTabViewItem
{
	[self willChangeValueForKey: @"displayMode"];
	
	[[NSUserDefaults standardUserDefaults] setObject: [NSNumber numberWithInt:
		displayMode= [[aTabViewItem label] isEqualTo: @"Library"]?TRWindowLibraryDisplayMode:TRWindowBrowseDisplayMode]
		forKey: TRWindowDisplayModeKey];

	if(displayMode==TRWindowLibraryDisplayMode)
	{
		[movieListingController unbind: @"contentArray"];
		[movieListingController bind: @"contentArray" toObject: controller withKeyPath: @"library" options: nil];

		//[movieListingController setContent: [controller library]];
		
		if(movieListingTimer) [movieListingTimer invalidate];
	}
	else
	{
		[movieListingController unbind: @"contentArray"];
		[movieListingController bind: @"contentArray" toObject: [controller client] withKeyPath: @"movies" options: nil];
		//[movieListingController setContent: [[controller client] movies]];
		
		movieListingTimer= [NSTimer scheduledTimerWithTimeInterval: 60 
			target: self
			selector: @selector(reloadMovieData:)
			userInfo: nil
			repeats: YES];
	}
	[self didChangeValueForKey: @"displayMode"];
}

@end

@implementation TRWindowController (NSSplitViewDelegate)

- (void)splitView:(NSSplitView *)sender
    resizeSubviewsWithOldSize:(NSSize)oldSize
{
    NSView *lhsView= [[mainSplitView subviews] objectAtIndex: 0];
    NSView *rhsView= [[mainSplitView subviews] objectAtIndex: 1];
    if(isCollapsed && !collapseInProgress)
        [lhsView setFrameSize:
            NSMakeSize([mainSplitView frame].size.width,[mainSplitView frame].size.height)];
    else if([rhsView frame].size.width>386.0 || oldSize.width<=[mainSplitView frame].size.width || 
            collapseInProgress)
        [lhsView setFrameSize:
            NSMakeSize([lhsView frame].size.width,[mainSplitView frame].size.height)];
    else
    {
        float difference= oldSize.width-[mainSplitView frame].size.width;
        [lhsView setFrameSize:
            NSMakeSize([lhsView frame].size.width-difference,[mainSplitView frame].size.height)];
        [rhsView setFrameOrigin: NSMakePoint([rhsView frame].origin.x-difference,0.0)];
    }
    [rhsView setFrameSize:
        NSMakeSize([mainSplitView frame].size.width-[lhsView frame].size.width-
                   [mainSplitView dividerThickness],[mainSplitView frame].size.height)];
}

- (float)splitView:(NSSplitView *)sender
    constrainSplitPosition:(float)aProposedPosition
    ofSubviewAt:(int)anOffset
{
    float width;
    if(aProposedPosition<180.0) return 180.0;
    if(aProposedPosition>(width= [mainSplitView frame].size.width-386.0))
        return width;
    return aProposedPosition;
}

- (void)splitViewDidResizeSubviews:(NSNotification *)aNotification
{
    [[NSUserDefaults standardUserDefaults]
        setObject: [NSNumber numberWithFloat: [[[mainSplitView subviews] objectAtIndex: 0] frame].size.width]
        forKey: TRMovieListWidthKey];
    if(!isCollapsed)
        [[NSUserDefaults standardUserDefaults]
            setObject: [NSNumber numberWithFloat: [[[mainSplitView subviews] objectAtIndex: 1] frame].size.width]
            forKey: TRAuxiliaryViewWidthKey];
}

@end