//
//  TRBroadcasting.m
//  Tryst
//
//  Created by Francisco Tolmasky on Mon May 10 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//


#import <stdlib.h>
#import <stdio.h>
#import <unistd.h>
#import <sys/socket.h>
#import <netinet/in.h>
#import <unistd.h>
#import <arpa/inet.h>

#import <AppKit/AppKit.h>

#import "TRMovie.h"
#import "TRMovieStreamer.h"
#import "TRBroadcasting.h"


NSString * const kTRBroadcastServerProperty=		@"Broadcast Server Property";
NSString * const kTRBroadcastSocketProperty=		@"Broadcast Socket Property";
NSString * const kTRBroadcastConnectionProperty=	@"Broadcast Connection Property";

NSString * const TRBroadcastServerInformationDidChangeNotification= @"Broadcast Server Information Did Change";
NSString * const TRBroadcastClientInformationDidChangeNotification= @"Broadcast Client Information Did Change";

@implementation TRBroadcastNode

- (id)init
{
	self= [super init];
	if(self) uniqueId= [[[NSHost currentHost] address] copy];
	return self;
}

- (NSString *)uniqueId
{
	return uniqueId;
}

- (void)dealloc
{
	[uniqueId release];
	[super dealloc];
}

@end

@implementation TRBroadcastServer

- (id)init
{
	self= [super init];
	if(self)
	{
		broadcast.timeRemaining= 0;
		broadcast.connectionLimit= 0;
		
		clients= [[NSMutableDictionary alloc] init];
		subscriptions= [[NSMutableDictionary alloc] init];
		
		// Create a psuedo-unique multicast address, meaning no tryst server will ever conflict with another.
		// However, another person or program might.
		
		if([[self uniqueId] rangeOfString: @":"].location!=NSNotFound) //ipv6
		{
			NSString *last= [[self uniqueId] substringFromIndex: [[self uniqueId] rangeOfString: @":" options: NSBackwardsSearch].location+1];
			int index= [last length]>1?[last length]-2:0;
			NSString *right= [NSString stringWithFormat: @"0x%@",[last substringFromIndex: index]];
			NSString *left= [NSString stringWithFormat: @"0x%@",[last substringWithRange: NSMakeRange(0,index)]];
		
			multicast= [[NSString stringWithFormat: @"239.255.%d.%d", strtol([left cString],nil,16), strtol([right cString],nil,16)] retain];
		}
		else
		{
			NSArray *components= [[self uniqueId] componentsSeparatedByString: @"."];
			multicast= [[NSString stringWithFormat: @"239.255.%@.%@", [components objectAtIndex: 2], [components objectAtIndex: 3]] retain];
		}
		
		isPlaying= NO;
	}
	return self;
}

- (BOOL)isPlaying
{
	return isPlaying;
}

- (TRMutableMovie *)movie
{
	return broadcast.movie;
}

- (NSTimeInterval)timeRemaining
{
	return broadcast.timeRemaining;
}

- (unsigned)connectionLimit
{
	return broadcast.connectionLimit;
}

- (unsigned)totalConnections
{
	return [subscriptions count];
}

- (void)broadcastMovie:(TRMutableMovie *)aMovie
	timeRemaining:(NSTimeInterval)aTimeInterval
	connectionLimit:(unsigned)aConnectionLimit
	password:(NSString *)aPassword
{
	broadcast.movie= [aMovie retain];
	broadcast.timeRemaining= aTimeInterval;
	broadcast.connectionLimit= aConnectionLimit;
	broadcast.password= [aPassword copy];
	
	[broadcast.movie setValue: self forProperty: kTRBroadcastServerProperty];
	
	int port;
	
	struct sockaddr * address;
	
	socket= [[NSSocketPort alloc] init];
	
	address= (struct sockaddr *)[[socket address] bytes];
	
	if(address->sa_family==AF_INET) port= ((struct sockaddr_in *)address)->sin_port;
	else if(address->sa_family==AF_INET6) port= ((struct sockaddr_in6 *)address)->sin6_port;

	service= [[NSNetService alloc] initWithDomain: @"" type: @"_tryst._tcp" name: [NSString stringWithFormat: @"Tryst Broadcast: %@", [self uniqueId]] port: port];
	[service publish];
	
	connection= [[NSConnection alloc] initWithReceivePort: socket sendPort: socket];
	
	//[connection setIndependentConversationQueueing: YES];
	[connection setRootObject: self];
	
	timer= [NSTimer scheduledTimerWithTimeInterval: 60
		target: self
		selector: @selector(updateTimeRemainingWithTimer:)
		userInfo: nil
		repeats: YES];
}

- (void)updateTimeRemainingWithTimer:(NSTimer *)aTimer
{
	if(!(broadcast.timeRemaining-= 60))
	{
		isPlaying= YES;
		
		NSString *playTitle= [broadcast.movie valueForProperty: kTRMovieTitleProperty];
		if(!playTitle) playTitle= [[[broadcast.movie filePath] lastPathComponent] stringByDeletingPathExtension];
		
		[[[NSApp delegate] windowController] dontAllowManualCancel];
		
		NSDictionary *pairs= [NSDictionary
		dictionaryWithObjects: [NSArray arrayWithObjects:
			[NSString stringWithFormat: @"file://%@",[[self movie] filePath]],
			[NSArray arrayWithObject: [NSString stringWithFormat: @":sout=#duplicate{dst=display,dst=std{access=udp,mux=ts,url=\"%@:1234\"}}", multicast]],
			nil]
		forKeys: [NSArray arrayWithObjects:
			@"ITEM_URL",
			@"ITEM_OPTIONS",
			nil]];
		
		[TRMovieStreamer streamWithPairs: pairs];
		
		TRBroadcastClient *client;
		NSEnumerator *enumerator= [subscriptions objectEnumerator];
		while(client= [enumerator nextObject])
		{
			//NS_DURING
			[client receiveStreamWithMulticastAddress: multicast name: playTitle];
			/*NS_HANDLER
			NSString *lostId= [[subscriptions allKeysForObject: client] objectAtIndex: 0];
			[clients removeObjectForKey: lostId];
			[subscriptions removeObjectForKey: lostId];
			NS_ENDHANDLER*/
		}
	}
	
	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastServerInformationDidChangeNotification
		object: self];
}

- (BOOL)isPasswordProtected
{
	return broadcast.password?YES:NO;
}

- (void)registerClient:(TRBroadcastClient *)aClient
{
	[clients setObject: aClient forKey: [aClient uniqueId]];
}

- (TRBroadcastResponse)subscribeClient:(TRBroadcastClient *)aClient withPassword:(NSString *)aPassword;
{
	if(broadcast.password && ![broadcast.password isEqualTo: aPassword]) return TRBroadcastPasswordInvalidResponse;
	if(broadcast.connectionLimit && broadcast.connectionLimit<=[self totalConnections]) return TRBroadcastConnectionLimitResponse;
	
	[subscriptions setObject: aClient forKey: [aClient uniqueId]];

	if(isPlaying) [aClient receiveStreamWithMulticastAddress: multicast name: @""];

	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastServerInformationDidChangeNotification
		object: self];
	
	return TRBroadcastSuccessfulResponse;
}

- (void)unsubscribeClient:(TRBroadcastClient *)aClient
{
	if(isPlaying) [aClient stopReceiving];
	
	[subscriptions removeObjectForKey: [aClient uniqueId]];
	
	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastServerInformationDidChangeNotification
		object: self];
}

- (void)unregisterClient:(TRBroadcastClient *)aClient
{	
	[clients removeObjectForKey: [aClient uniqueId]];
	
	TRBroadcastClient *subscription= [subscriptions objectForKey: [aClient uniqueId]];
	if(subscription)
	{
		if(isPlaying) [aClient stopReceiving];
		
		[subscriptions removeObjectForKey: [aClient uniqueId]];
	}
	
	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastServerInformationDidChangeNotification
		object: self];
}

- (void)stop
{
	if(!service) return;

	if([timer isValid]) [timer invalidate];

	[service stop];
	[service release];
	
	TRBroadcastClient *client;
	NSEnumerator *enumerator= [clients objectEnumerator];
	while(client= [enumerator nextObject]) [client closeConnectionWithServer: self];
	
	[connection registerName: nil];
	[socket invalidate];
	[connection invalidate];
	
	[socket release];
	[connection release];
	
	[broadcast.movie removeValueForProperty: kTRBroadcastServerProperty];
	[broadcast.movie release];
	
	service= nil;
	isPlaying= NO;
}

@end

@implementation TRBroadcastClient 

- (id)init
{
	self= [super init];
	if(self)
	{
		movies= [[NSMutableArray alloc] init];
		
		browser= [[NSNetServiceBrowser alloc] init];
		[browser setDelegate: self];
		[browser searchForServicesOfType: @"_tryst._tcp." inDomain: @""];
			
	}
	return self;
}

- (void)netServiceBrowser:(NSNetServiceBrowser *)aNetServiceBrowser didFindService:(NSNetService *)aNetService moreComing:(BOOL)moreComing
{
	[aNetService retain];
	[aNetService setDelegate: self];
	[aNetService resolve];
}

- (void)netServiceDidResolveAddress:(NSNetService *)sender
{
	int port;
	
	char buffer[256];
	NSString *hostString= @"";
	
	struct sockaddr * address= (struct sockaddr *)[[[sender addresses] objectAtIndex: 0] bytes];

	if(address->sa_family==AF_INET)
	{
		if(inet_ntop(AF_INET, &((struct sockaddr_in *)address)->sin_addr, buffer, sizeof(buffer)))
		{
			hostString= [NSString stringWithCString: buffer];
			port= ((struct sockaddr_in *)address)->sin_port;
		}
	}
	else if(address->sa_family==AF_INET6)
	{
		port= ((struct sockaddr_in6 *)address)->sin6_port;
	}
	
	[sender stop];
	[sender release];
	
	NSSocketPort	*socket= [[NSSocketPort alloc] initRemoteWithTCPPort: port host: hostString];
	NSConnection	*connection= [[NSConnection alloc] initWithReceivePort: nil sendPort: socket];
	
	id server= [connection rootProxy];

	[server setProtocolForProxy: @protocol(TRBroadcastServerProtocol)];
	
	// Make sure this is not us
	if([[server uniqueId] isEqualTo: [self uniqueId]]) return;
	
	[server registerClient: self];
	
	TRMutableMovie *movie= [(TRBroadcastServer *)server movie];

	[movie setValue: server forProperty: kTRBroadcastServerProperty];
	[movie setValue: socket forProperty: kTRBroadcastSocketProperty];
	[movie setValue: connection forProperty: kTRBroadcastConnectionProperty];

	[movies addObject: movie];

	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastClientInformationDidChangeNotification
		object: self];
	
	// For some reason can't do this?  ArrayController dies on switch.
	//[socket release];
	//[connection release];
	//[movie release];
}

- (NSArray *)movies
{
	return movies;
}

- (void)closeConnectionForMovie:(TRMovie *)aMovie
{
	TRBroadcastServer *server= [aMovie valueForProperty: kTRBroadcastServerProperty];
	NSSocketPort *socket= [aMovie valueForProperty: kTRBroadcastSocketProperty];
	NSConnection *connection= [aMovie valueForProperty: kTRBroadcastConnectionProperty];

	if(connectedServer==server) connectedServer= nil;
	
	[server unregisterClient: self];

	[connection registerName: nil];
	[socket invalidate];
	[connection invalidate];
}

- (void)close
{
	int i= 0,
		length= [movies count];
	for(;i<length;++i)
	{
		[self closeConnectionForMovie: [movies objectAtIndex: 0]];
		[movies removeObjectAtIndex: 0];
	}
}

- (void)closeConnectionWithServer:(TRBroadcastServer *)aServer
{	
	int i= 0,
		length= [movies count];
	
	NSString *serverUniqueId= [aServer uniqueId];
		
	for(;i<length;++i) if([[(TRBroadcastServer *)[[movies objectAtIndex: i] valueForProperty: kTRBroadcastServerProperty] uniqueId] isEqualTo: serverUniqueId])
	{
		[self closeConnectionForMovie: [movies objectAtIndex: i]];
		[movies removeObjectAtIndex: i];
		break;
	}

	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastClientInformationDidChangeNotification
		object: self];
}

- (void)receiveStreamWithMulticastAddress:(NSString *)multicastAddress name:(NSString *)movieName
{
	[TRMovieStreamer streamWithPairs: [NSDictionary dictionaryWithObjects: 
		[NSArray arrayWithObjects:
			movieName,
			[NSString stringWithFormat: @"udp://@%@", multicastAddress],
			nil]
		forKeys: [NSArray arrayWithObjects: @"ITEM_NAME",@"ITEM_URL",nil]]];
}

- (void)unsubscribe
{	
	if(!connectedServer) return;
	
	[connectedServer unsubscribeClient: self];

	connectedServer= nil;
	
	[[NSNotificationCenter defaultCenter]
		postNotificationName: TRBroadcastClientInformationDidChangeNotification
		object: self];
}

- (void)setServer:(TRBroadcastServer *)aServer
{
	connectedServer= aServer;
}

- (TRBroadcastServer *)server
{
	return connectedServer;
}

- (void)stopReceiving
{
	[[[NSApp delegate] getControls] stop: nil];
	[[NSApp delegate] cleanUpSubscribe];
}

@end
