//
//  TRMovie.m
//  Tryst
//
//  Created by Francisco Tolmasky on Fri Nov 28 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "NDAlias.h"

#import "TRMultiValue.h"
#import "TRMovie.h"
#import "TRBroadcasting.h"

NSString * const kTRMovieTitleProperty=     @"Movie Title Property";
NSString * const kTRMovieYearProperty=      @"Movie Year Property";
NSString * const kTRMovieGenreProperty=     @"Movie Genre Property";
NSString * const kTRMovieCoverArtProperty=  @"Movie Cover Art Property";
NSString * const kTRMovieFileProperty=		@"Movie File Property";

NSString * const kTRMovieCastProperty=      @"Movie Cast Property";
NSString * const kTRMovieCrewProperty=      @"Movie Crew Property";

NSString * const TRMoviesPboardType=		@"TRMovies";

static NSArray *TRMovieFileTypes;

@implementation TRMovie

+ (void)initialize
{
    if(self==[TRMovie class])
    {
		[TRMovie setVersion: 1];
            
        TRMovieFileTypes= [[NSArray alloc] initWithObjects: @"mov", @"MOV", @"'MooV'", @"mpeg", @"mpg", @"avi", nil];
    }
}

+ (NSArray *)movieFileTypes
{
    return TRMovieFileTypes;
}

- (id)init
{
    self= [super init];
    if(self)
    {
        properties= [[NSMutableDictionary allocWithZone: [self zone]] init];
    
        /*[self setValue: @"Untitled" forProperty: kTRMovieTitleProperty];
        [self setValue: @"2003" forProperty: kTRMovieYearProperty];
        [self setValue: @"Action" forProperty: kTRMovieGenreProperty];
        
        TRMutableMultiValue *cast= [[TRMutableMultiValue alloc] init];
        [cast addValue: @"Person 1" withLabel: @"actor"];
        [cast addValue: @"Person 2" withLabel: @"actor"];
        [cast addValue: @"Person 3" withLabel: @"actor"];
		
        [self setValue: cast forProperty: kTRMovieCastProperty];*/
    }
    return self;
}

- (id)initWithMovie:(TRMovie *)aMovie
{
    self= [super init];
    if(self)
        properties= [[aMovie /*->properties*/ valueForKey: @"properties"] mutableCopyWithZone: [self zone]];
	return self;
}

- (id)initWithCoder:(NSCoder *)aCoder
{
    self= [super init]; //NSObject does not conform to NSCoding.
	
	if(self)
	{
		if ( [aCoder allowsKeyedCoding] )
			properties= [[aCoder decodeObjectForKey: @"TRProperties"] retain];
		else
			//[NSException raise: NSInvalidArchiveOperationException format: @"TRMovie only supports NSKeyedArchiver coders."];
			properties= [[aCoder decodeObject] retain];
		
		// Make sure file still exists or we at least still know where it is.
		NSData *aliasData= [self valueForProperty: kTRMovieFileProperty];
		if(aliasData && ![[NDAlias aliasWithData: aliasData] path]) [properties removeObjectForKey: kTRMovieFileProperty]; //[self removeValueForProperty: ?
	}
	
    return self;
}

- (void)dealloc
{
    [properties release];
    [super dealloc];
}

- (void)encodeWithCoder:(NSCoder *)aCoder
{
    //[super encodeWithCoder: aCoder];
    if ( [aCoder allowsKeyedCoding]/*[aCoder isKindOfClass: [NSKeyedArchiver class]]*/ )
		[aCoder encodeObject: properties forKey: @"TRProperties"];
	else
		[aCoder encodeObject: properties];
}

- (id)replacementObjectForPortCoder:(NSPortCoder *)aCoder
{
	if ([aCoder isBycopy]) return self;
	return [super replacementObjectForPortCoder: aCoder];
}

- (id)valueForUndefinedKey:(NSString *)aKey
{
    return [properties objectForKey: aKey];
}

- (id)valueForProperty:(NSString *)aProperty
{
    return [properties objectForKey: aProperty];
}

- (NSEnumerator *)propertyEnumerator
{
    return [[properties allKeys] reverseObjectEnumerator];
}

- (NSData *)imageData
{
    return (NSData *)[properties objectForKey: kTRMovieCoverArtProperty];
}

- (NSString *)filePath
{
	NSData *aliasData= [self valueForProperty: kTRMovieFileProperty];
	return aliasData?[[NDAlias aliasWithData: aliasData] path]:nil;
}

- (id)copyWithZone:(NSZone *)aZone
{
	return [self retain];//[(TRMovie *)[TRMovie allocWithZone: aZone] initWithMovie: self];
}

- (id)mutableCopyWithZone:(NSZone *)aZone
{
    return [(TRMovie *)[TRMutableMovie allocWithZone: aZone] initWithMovie: self];
}

@end

@implementation TRMutableMovie

+ (void)initialize
{
	if(self==[TRMutableMovie class])
	{
		[TRMutableMovie setVersion: 1];
		
		[TRMutableMovie setKeys: [NSArray arrayWithObjects: kTRMovieTitleProperty, kTRMovieYearProperty, kTRMovieGenreProperty, nil]
            triggerChangeNotificationsForDependentKey: @"self"];
	}
}

- (id)isEditable
{
	return [self valueForKey: kTRBroadcastServerProperty]?nil:self;
}

- (void)setValue:(NSString *)aValue forProperty:(NSString *)aProperty
{
    // Implementing manual observer notification
	if(aProperty==kTRBroadcastServerProperty) [self willChangeValueForKey: @"isEditable"];
	else [self willChangeValueForKey: aProperty];
    [properties setObject: aValue forKey: aProperty];
	if(aProperty==kTRBroadcastServerProperty) [self didChangeValueForKey: @"isEditable"];
	else [self didChangeValueForKey: aProperty];
}

- (void)removeValueForProperty:(NSString *)aProperty
{
    // Implementing manual observer notification
	if(aProperty==kTRBroadcastServerProperty) [self willChangeValueForKey: @"isEditable"];
	else [self willChangeValueForKey: aProperty];
	[properties removeObjectForKey: aProperty];
	if(aProperty==kTRBroadcastServerProperty) [self didChangeValueForKey: @"isEditable"];
	else [self didChangeValueForKey: aProperty];
}

- (void)removeAllProperties
{
    [properties removeAllObjects];
}

- (void)setImageData:(NSData *)imageData
{
	[self setValue: [[imageData copy] autorelease] forProperty: kTRMovieCoverArtProperty];
}

- (void)setFilePath:(NSString *)filePath
{
	[self setValue: [[NDAlias aliasWithPath: filePath] data] forProperty: kTRMovieFileProperty];
}

@end