//
//  TRMovieFileView.m
//  Tryst
//
//  Created by Francisco Tolmasky on Mon Mar 08 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMovie.h"
#import "NSArray+TRAdditions.h"

#import "ShadeRect.h"
#import "NSString+TRDrawingAdditions.h"

#import "TRMovieInspector.h"
#import "TRRolloverButton.h"
#import "TRMovieFileView.h"


@interface TRMovieInspector (TRMovieFileViewFriend)
- (void)didChangeTextAtLocation:(unsigned)aLocation;
- (void)setMovieTitle:(NSString *)aMovieTitle;
- (NSString *)movieTitle;
@end

@implementation TRMovieInspector (TRMovieFileViewAdditions)

- (void)setMovieTitle:(NSString *)aMovieTitle
{
	NSRange effectiveRange;
	
	NSTextStorage *textStorage= [self textStorage];
	
	[textStorage attribute: kPropertyAttributeName atIndex: 3 effectiveRange: &effectiveRange];
    
    NSAttributedString *a_str= [[NSAttributedString alloc] initWithString: aMovieTitle attributes:
        [self typingAttributesForProperty: kTRMovieTitleProperty identifier: @"" placeholderString: @"Movie Title" font: [NSFont  boldSystemFontOfSize: 14]]];
    
    [self setSelectedRange: NSMakeRange(0,0)];
    [textStorage replaceCharactersInRange: effectiveRange withAttributedString: a_str];
    [self setSelectedRange: NSMakeRange(effectiveRange.location,0)];
    
	[a_str release];
	
    [self didChangeTextAtLocation: effectiveRange.location];
}

- (NSString *)movieTitle
{
	NSRange effectiveRange;
	
	NSTextStorage *textStorage= [self textStorage];
	
	[textStorage attribute: kPropertyAttributeName atIndex: 3 effectiveRange: &effectiveRange];
	
	return [[textStorage string] substringWithRange: effectiveRange];
}

@end

@implementation TRMovieFileView

- (id)initWithFrame:(NSRect)aFrame
{
	self= [super initWithFrame: aFrame];
	if(self)
	{
		//flags.editable= NO;
		flags.dragging= NO;
	
		[self registerForDraggedTypes: [NSArray arrayWithObject: NSFilenamesPboardType]];
	
		revealButton= [[TRRolloverButton alloc] initWithFrame: NSMakeRect(aFrame.size.width-22.0,(aFrame.size.height-18.0)/2,18.0,18.0)];
		
		[revealButton setImage: [NSImage imageNamed: @"Reveal"]];
		[revealButton setTarget: self];
		[revealButton setAction: @selector(reveal:)];
		[revealButton setToolTip: @"Reveal file in finder"];
        [revealButton setAutoresizingMask: NSViewMinXMargin];
		
		[self addSubview: revealButton];
	}
	return self;
}

- (void)dealloc
{
	[revealButton release];
	
	[super dealloc];
}

- (NSMenu *)menuForEvent:(NSEvent *)theEvent
{
	NSMenu *menu= [[NSMenu alloc] initWithTitle: @"File Options"];
	
	NSPasteboard *pasteboard= [NSPasteboard generalPasteboard];
	NSArray *filteredFilenames;
	
	if([model filePath])
	{
        [menu addItemWithTitle: @"Cut" action: @selector(cut:) keyEquivalent: @""];
        [menu addItemWithTitle: @"Copy" action: @selector(copy:) keyEquivalent: @""];
        [menu addItemWithTitle: @"Clear" action: @selector(clear:) keyEquivalent: @""];
	}
	if( [[pasteboard types] containsObject: NSFilenamesPboardType] &&
		[filteredFilenames= [[pasteboard propertyListForType: NSFilenamesPboardType] pathsAndSubpathsMatchingExtensions: [TRMovie movieFileTypes]] count])
		[menu addItemWithTitle: @"Paste" action: @selector(paste:) keyEquivalent: @""];

	return menu;
}

- (void)viewWillMoveToWindow:(NSWindow *)newWindow
{
    NSWindow *oldWindow= [self window];
    if (oldWindow)
    {
        NSNotificationCenter *notificationCenter= [NSNotificationCenter defaultCenter];
        [notificationCenter removeObserver: self name: NSWindowDidBecomeKeyNotification object: oldWindow];
        [notificationCenter removeObserver: self name: NSWindowDidResignKeyNotification object: oldWindow];
    }
    if (newWindow)
    {
        // Register for window becoming and resigning key notifications (The raised editor will draw different if its key, then when it is not key)
        NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
        [notificationCenter addObserver: self selector: @selector(windowKeyStateChangedNotification:) name: NSWindowDidBecomeKeyNotification object: newWindow];
        [notificationCenter addObserver: self selector: @selector(windowKeyStateChangedNotification:) name: NSWindowDidResignKeyNotification object: newWindow];
    }
}

- (void)setModel:(TRMutableMovie *)aModel
{
	model= aModel;
	
	[self setNeedsDisplay: YES];
}

- (TRMutableMovie *)model
{
	return model;
}

- (void)setMovieInspector:(TRMovieInspector *)aMovieInspector
{
	movieInspector= aMovieInspector;
	
	[self bind: @"model" toObject: movieInspector withKeyPath: @"model" options: nil];
	[self bind: @"hidden" toObject: movieInspector withKeyPath: @"model" options: [NSDictionary dictionaryWithObject: NSIsNilTransformerName forKey: @"NSValueTransformerName"]];

	[revealButton bind: @"hidden" toObject: movieInspector withKeyPath: [NSString stringWithFormat: @"model.%@", kTRMovieFileProperty] options: [NSDictionary dictionaryWithObject: NSIsNilTransformerName forKey: @"NSValueTransformerName"]];
}

- (TRMovieInspector *)movieInspector
{
	return movieInspector;
}

- (void)updateModelWithFilePath:(NSString *)aPath
{
	[self setNeedsDisplay: YES];
			
	if(aPath)
	{
		//[model setFile: [[[NSFileWrapper alloc] initSymbolicLinkWithDestination: aPath] autorelease]];
		[model setFilePath: aPath];
		
		NSString *title;
		
		[[self window] makeFirstResponder: self];
		
		if(![[movieInspector movieTitle] isEqual: title= [[aPath lastPathComponent] stringByDeletingPathExtension]])
			NSBeginAlertSheet(
				@"Update Movie Title",
				@"Yes", @"No", nil,
				[self window],
				self,
				@selector(sheetDidEnd: returnCode: contextInfo:),
				NULL,
				(void *)([title retain]),
				[NSString stringWithFormat: @"Update Movie Title to \"%@\"?", title]);
	}
	else
	{
		[model removeValueForProperty: kTRMovieFileProperty];
		[[self window] makeFirstResponder: nil];
	}
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if(returnCode==NSAlertDefaultReturn)
	{
		if(![movieInspector isEditable]) [movieInspector setEditable: YES];
		[movieInspector setMovieTitle: (NSString *)contextInfo];
		[[self window] makeFirstResponder: movieInspector];
	}
	
	[(NSString *)contextInfo release];
}

- (void)browse:(id)sender
{
    [[NSOpenPanel openPanel]
        beginSheetForDirectory: nil
        file: nil
        types: [TRMovie movieFileTypes]
        modalForWindow: [self window]
        modalDelegate: self
        didEndSelector: @selector(openPanelDidEnd: returnCode: contextInfo:)
        contextInfo: nil];
}

- (void)openPanelDidEnd:(NSOpenPanel *)openPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if(returnCode==NSCancelButton) return;
    
	[openPanel orderOut: self];
	
	[self updateModelWithFilePath: [[[openPanel URLs] objectAtIndex: 0] path]];
	
	[self setNeedsDisplay: YES];
}

- (void)windowKeyStateChangedNotification:(NSNotification *)aNotification
{
	if(self==[[self window] firstResponder]) [self setNeedsDisplay: YES];
}

- (BOOL)isFirstResponder
{
	return self==[[self window] firstResponder];
}

- (BOOL)acceptsFirstResponder
{
	return [model filePath]?YES:NO; // avoid warnings >(.
}

- (BOOL)becomeFirstResponder
{
	if([super becomeFirstResponder])
	{
		[revealButton setIlluminated: YES];
		[self setNeedsDisplay: YES];
		return YES;
	}
	return NO;
}

- (BOOL)resignFirstResponder
{
	if([super resignFirstResponder])
	{
		[revealButton setIlluminated: NO];
		[self setNeedsDisplay: YES];
		return YES;
	}
	return NO;
}

- (void)drawRect:(NSRect)clipRect
{
	if([model filePath])
	{
		NSColor *filenameColor= [NSColor blackColor],
				*filesizeColor= [NSColor grayColor];
		
		NSWindow *window= [self window];
		
		if(self==[window firstResponder])
		{
			CGRGB start, end;
			
			filenameColor= filesizeColor= [NSColor whiteColor];
			
			if([window isKeyWindow])
			{
				start=  CGRGBMake(0.0f, 77.0/255.0f, 208.0/255.0f);
				end= CGRGBMake(68.0/255.0f, 146.0/255.0f, 230.0/255.0f);
			}
			else
			{
				start= CGRGBMake(102.0/255.0f, 102.0/255.0f, 102.0/255.0f);
				end= CGRGBMake(151.0/255.0f, 151.0/255.0f, 151.0/255.0f);
			}
			
			HIRect bounds= CGRectMake([self bounds].origin.x,[self bounds].origin.x,[self bounds].size.width,[self bounds].size.height);
			ShadeRectColor( &start, &end, &bounds, [[NSGraphicsContext currentContext] graphicsPort] );
		}
		
		if(flags.dragging)
		{
			float   capRadius= 5;
	
			NSRect insideRect= NSMakeRect(4.0,4.0,[self frame].size.width-capRadius-10.0,[self frame].size.height-8.0);
		
			NSBezierPath *path= [NSBezierPath bezierPath];
			
			[path setLineWidth: 2.0];
		
			[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(insideRect)+2.0,NSMinY(insideRect)+capRadius) radius: capRadius startAngle: 270.0f endAngle: 360.0f];// clockwise: YES];
			[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(insideRect)+2.0,NSMaxY(insideRect)-capRadius) radius: capRadius startAngle: 0.0f endAngle: 90.0f];// clockwise: YES];
			[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(insideRect)+capRadius,NSMaxY(insideRect)-capRadius) radius: capRadius startAngle: 90.0f endAngle: 180.0f];
			[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(insideRect)+capRadius,NSMinY(insideRect)+capRadius) radius: capRadius startAngle: 180.0f endAngle: 270.0f];
	
			[path closePath];
		
			if(self==[window firstResponder])
			{
				[[NSColor colorWithCalibratedRed: 1.0f green: 1.0f blue: 1.0f alpha: 1.0f] set];
				
				[path stroke];
			}
			else
			{
				[[NSColor colorWithCalibratedRed: 198.0/255.0f green: 214.0/255.0f blue: 242.0/255.0f alpha: 1.0f] set];
				
				[path fill];
				
				[[NSColor colorWithCalibratedRed: 31.0/255.0f green: 92.0/255.0f blue: 207.0/255.0f alpha: 1.0f] set];
				
				[path stroke];
			}
		}
	
		[[[NSWorkspace sharedWorkspace] iconForFile: [model filePath]] compositeToPoint: NSMakePoint(4.0,4.0) operation: NSCompositeSourceOver];
		
		int bytes= [[[[NSFileManager defaultManager] fileAttributesAtPath: [model filePath] traverseLink: YES] objectForKey: NSFileSize] intValue];
		
		NSString *fileSize= [NSString stringWithFileSize: bytes];
		
		NSMutableParagraphStyle *truncatingParagraphStyle= [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
        [truncatingParagraphStyle setLineBreakMode: NSLineBreakByTruncatingMiddle];
		
		NSAttributedString *filenameString= [[NSAttributedString alloc] initWithString: [[model filePath] lastPathComponent] 
			attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 12.0], filenameColor, truncatingParagraphStyle, nil]
			forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, NSParagraphStyleAttributeName, nil]]];
		NSAttributedString *filesizeString= [[NSAttributedString alloc] initWithString: fileSize
			attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 9.0], filesizeColor, nil]
            forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]]];
		
		[filenameString drawInRect: NSMakeRect(36.0,[self frame].size.height-[filenameString size].height-6.0,[self frame].size.width-62.0,[filenameString size].height)];
		[filesizeString drawInRect: NSMakeRect(36.0,6.0,[self frame].size.width-62.0,[filesizeString size].height)];
		
		NSBezierPath *path= [NSBezierPath bezierPath];
		[path moveToPoint: NSMakePoint(0.0,[self frame].size.height)];
		[path lineToPoint: NSMakePoint([self frame].size.width,[self frame].size.height)];
		
		[[NSColor lightGrayColor] set];
		
		[path stroke];
	}
	else
	{
		float   capRadius= 8;
	
		NSRect insideRect= NSMakeRect(4.0,4.0,[self frame].size.width-capRadius-10.0,[self frame].size.height-8.0);
		
		NSBezierPath *path= [NSBezierPath bezierPath];
		
		[path setLineWidth: 2.0];
		
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(insideRect)+2.0,NSMinY(insideRect)+capRadius) radius: capRadius startAngle: 270.0f endAngle: 360.0f];// clockwise: YES];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(insideRect)+2.0,NSMaxY(insideRect)-capRadius) radius: capRadius startAngle: 0.0f endAngle: 90.0f];// clockwise: YES];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(insideRect)+capRadius,NSMaxY(insideRect)-capRadius) radius: capRadius startAngle: 90.0f endAngle: 180.0f];
		[path appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(insideRect)+capRadius,NSMinY(insideRect)+capRadius) radius: capRadius startAngle: 180.0f endAngle: 270.0f];
	
		[path closePath];
	
		if(flags.dragging)
		{
			[[NSColor colorWithCalibratedRed: 198.0/255.0f green: 214.0/255.0f blue: 242.0/255.0f alpha: 1.0f] set];
			
			[path fill];
			
			[[NSColor colorWithCalibratedRed: 31.0/255.0f green: 92.0/255.0f blue: 207.0/255.0f alpha: 1.0f] set];
				
			[path stroke];
		}
		else
		{
			float pattern[]= {8.0, 4.0};
				
			[path setLineDash: pattern count: 2 phase: 0.0];
				
			[[NSColor lightGrayColor] set];
		
			[path stroke];
		}
		
		NSAttributedString *dragFileString= [[NSAttributedString alloc] initWithString: NSLocalizedString(@"Drag Movie File Here",@"Drag File")
			attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont boldSystemFontOfSize: 12.0], flags.dragging?[NSColor blackColor]:[NSColor lightGrayColor], nil] 
			forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]]];
	
		[dragFileString drawAtPoint: NSMakePoint(([self frame].size.width-[dragFileString size].width)/2,([self frame].size.height-[dragFileString size].height)/2)];
	}
}

// Copy-Paste

- (void)reveal:(id)sender
{
	[[NSWorkspace sharedWorkspace]
		selectFile: [model filePath]
		inFileViewerRootedAtPath: [model filePath]];
}

- (void)clear:(id)sender
{
	[self updateModelWithFilePath: nil];
}

- (void)copy:(id)sender
{	
	NSPasteboard *pasteboard= [NSPasteboard generalPasteboard];
	[pasteboard declareTypes: [NSArray arrayWithObject: NSFilenamesPboardType] owner: self];
	[pasteboard setPropertyList: [NSArray arrayWithObject: [model filePath]] forType: NSFilenamesPboardType];
}

- (void)cut:(id)sender
{
	[self copy: self];
	
	[self clear: self];
}

- (void)paste:(id)sender
{
	NSPasteboard *pasteboard= [NSPasteboard generalPasteboard];
	NSString *firstPath;

	if( [[pasteboard types] containsObject: NSFilenamesPboardType] &&
		(firstPath= [[pasteboard propertyListForType: NSFilenamesPboardType] firstPathOrSubpathMatchingExtensions: [TRMovie movieFileTypes]]))
		[self updateModelWithFilePath: firstPath];
	else NSBeep();
}

// Dragging Source

- (void)mouseDown:(NSEvent *)theEvent
{
    if([theEvent clickCount]==2) [self browse: self]; 
}

- (void)mouseDragged:(NSEvent *)theEvent
{
	if(![model filePath]) return;
	
	NSPasteboard *pasteboard = [NSPasteboard pasteboardWithName: NSDragPboard];
    [pasteboard declareTypes: [NSArray arrayWithObject: NSFilenamesPboardType] owner: self];
    [pasteboard setPropertyList: [NSArray arrayWithObject: [model filePath]] forType: NSFilenamesPboardType];
	
	NSImage *dragImage= [[NSImage alloc] initWithSize: [self frame].size];
	
	[dragImage lockFocus];
	
	[[[NSWorkspace sharedWorkspace] iconForFile: [model filePath]] compositeToPoint: NSMakePoint(4.0,4.0) operation: NSCompositeSourceOver];
	
	int bytes= [[[[NSFileManager defaultManager] fileAttributesAtPath: [model filePath] traverseLink: YES] objectForKey: NSFileSize] intValue];
	
	NSString *fileSize= [NSString stringWithFileSize: bytes];
	
	NSMutableParagraphStyle *truncatingParagraphStyle= [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[truncatingParagraphStyle setLineBreakMode: NSLineBreakByTruncatingMiddle];
	
	NSAttributedString *filenameString= [[NSAttributedString alloc] initWithString: [[model filePath] lastPathComponent]
		attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 12.0], [NSColor blackColor], truncatingParagraphStyle, nil]
		forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, NSParagraphStyleAttributeName, nil]]];
	NSAttributedString *filesizeString= [[NSAttributedString alloc] initWithString: fileSize
		attributes: [NSDictionary dictionaryWithObjects: [NSArray arrayWithObjects: [NSFont systemFontOfSize: 9.0], [NSColor grayColor], nil]
		forKeys: [NSArray arrayWithObjects: NSFontAttributeName, NSForegroundColorAttributeName, nil]]];
	
	[filenameString drawInRect: NSMakeRect(36.0,[self frame].size.height-[filenameString size].height-6.0,[self frame].size.width-62.0,[filenameString size].height)];
	[filesizeString drawInRect: NSMakeRect(36.0,6.0,[self frame].size.width-62.0,[filesizeString size].height)];
	
	[dragImage unlockFocus];
	
	[self
        dragImage: dragImage
        at: NSMakePoint(0.0,0.0)
        offset: NSZeroSize
        event: theEvent
        pasteboard: pasteboard
        source: self
        slideBack: YES];
}

- (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag
{
	return NSDragOperationLink | NSDragOperationCopy | NSDragOperationDelete;
}

- (void)draggedImage:(NSImage *)anImage endedAt:(NSPoint)aPoint operation:(NSDragOperation)operation
{
	if(operation==NSDragOperationDelete) [self clear: self];
}

// Dragging Destination

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
    if( [sender draggingSource]==self || 
		![[[sender draggingPasteboard] types] containsObject: NSFilenamesPboardType] || 
		![[[sender draggingPasteboard] propertyListForType: NSFilenamesPboardType] firstPathOrSubpathMatchingExtensions: [TRMovie movieFileTypes]])
		return NSDragOperationNone;
    
	flags.dragging= YES;
	
    [self setNeedsDisplay: YES];
    
	return NSDragOperationCopy;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource]==self) return;
    
    flags.dragging= NO;
	
    [self setNeedsDisplay: YES];
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    return [sender draggingSource]!=self;//[NSImage canInitWithPasteboard: [sender flags.draggingPasteboard]];
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
	NSArray *filteredFilenames= [[[sender draggingPasteboard] propertyListForType: NSFilenamesPboardType] pathsAndSubpathsMatchingExtensions: [TRMovie movieFileTypes]];
    
	[self updateModelWithFilePath: [filteredFilenames objectAtIndex: 0]];
    
    flags.dragging= NO;
	
    [self setNeedsDisplay: YES];
	
	return YES;
}

@end
