//
//  TRImageView.m
//  Tryst
//
//  Created by Francisco Tolmasky on Tue Jan 27 2004.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRMovieInspector.h"
#import "TRMovieImageView.h"


@implementation TRMovieImageView

+ (void)initialize
{
    if(self==[TRMovieImageView class])
    {
        [TRMovieImageView setKeys: [NSArray arrayWithObject: @"image"] triggerChangeNotificationsForDependentKey: @"data"];
    }
}

- (id)initWithFrame:(NSRect)aFrame movieInspector:(TRMovieInspector *)aMovieInspector
{
    self= [super initWithFrame: aFrame];
    if(self)
    {
        dragging= NO;
        movieInspector= aMovieInspector;
    }
    return self;
}

- (void)dealloc
{
    //[data release];
    [super dealloc];
}

- (void)drawRect:(NSRect)clipRect
{
    [super drawRect: clipRect];

    if([[self window] isKeyWindow] && self==[[self window] firstResponder])// && [self isEditable])
    {
        [NSGraphicsContext saveGraphicsState];
        NSSetFocusRingStyle(NSFocusRingOnly);
        NSRectFill([self bounds]);
        [NSGraphicsContext restoreGraphicsState];
    }
    else if(dragging)
    {
        [[NSColor whiteColor] set];
        [[NSBezierPath bezierPathWithRect: [self bounds]] stroke];
        [[NSColor keyboardFocusIndicatorColor] set];
        [NSBezierPath strokeRect: [self bounds]];
        [NSBezierPath strokeRect: [self bounds]];
    }
}

// Copy/Cut/Paste

- (IBAction)copy:(id)sender
{
    NSPasteboard *pasteboard= [NSPasteboard generalPasteboard];
    
    [pasteboard declareTypes: [NSArray arrayWithObjects: NSTIFFPboardType, nil] owner: nil];
    if(![pasteboard setData: [[self image] TIFFRepresentation] forType: NSTIFFPboardType]) NSBeep();
}

- (void)cut:(id)sender
{
    [self copy: sender];
}

- (IBAction)paste:(id)sender
{
    if([NSImage canInitWithPasteboard: [NSPasteboard generalPasteboard]])
    {
        [self setImage: [[NSImage alloc] initWithPasteboard: [NSPasteboard generalPasteboard]]];
        [self sendAction: [self action] to: [self target]];
    }
    else NSBeep();
}

// Editability: to stay consistent

- (void)setEditable:(BOOL)flag
{
	if(!flag && self==[[self window] firstResponder]) [[self window] makeFirstResponder: nil];
	
	[super setEditable: flag];
}

// Drag Source

- (BOOL)acceptsFirstMouse:(NSEvent *)event
{
    return YES;//so source doesn't have to be the active window
}

- (BOOL)becomeFirstResponder
{
    [movieInspector setSelectedRange: NSMakeRange(0,0)];
    return YES;
}

- (BOOL)acceptsFirstResponder
{
    return YES;//[self isEditable];
}

// Open from file

- (void)openPanelDidEnd:(NSOpenPanel *)openPanel returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    if(returnCode==NSCancelButton) return;
    
    NSImage *image= [[NSImage alloc] initWithContentsOfURL: [[openPanel URLs] objectAtIndex: 0]];
    if([image isValid])
    {
        [self setImage: image];
        [image release];
        [self sendAction: [self action] to: [self target]];
    }
    else NSBeep();
}

- (void)mouseDown:(NSEvent *)theEvent
{
    if([theEvent clickCount]==2 && [self isEditable]) 
        [[NSOpenPanel openPanel]
            beginSheetForDirectory: nil
            file: nil
            types: [NSImage imageFileTypes]
            modalForWindow: [self window]
            modalDelegate: self
            didEndSelector: @selector(openPanelDidEnd: returnCode: contextInfo:)
            contextInfo: nil];
}

// Dragging

- (void)mouseDragged:(NSEvent*)theEvent
{
	if(![[movieInspector model] imageData]) return;
    //get the Pasteboard used for drag and drop operations
    NSPasteboard *dragPasteboard= [NSPasteboard pasteboardWithName: NSDragPboard];
    //create a new image for our semi-transparent drag image
    NSImage* dragImage= [[NSImage alloc] initWithSize: [[self image] size]]; 
    //add the image types we can send the data as(we'll send the actual data when it's requested)
    [dragPasteboard declareTypes: [NSArray arrayWithObject: NSTIFFPboardType] owner: self];
    [dragPasteboard addTypes: [NSArray arrayWithObject: NSPDFPboardType] owner: self];

    [dragImage lockFocus];//draw inside of our dragImage
    //draw our original image as 50% transparent
    [[self image] dissolveToPoint: NSZeroPoint fraction: .5];
    [dragImage unlockFocus];//finished drawing
    [dragImage setScalesWhenResized: YES];//we want the image to resize
    [dragImage setSize: [self bounds].size];//change to the size we are displaying
    //execute the drag
    [self dragImage: dragImage//image to be displayed under the mouse
        at: [self bounds].origin//point to start drawing drag image
        offset: NSZeroSize//no offset, drag starts at mousedown location
        event: theEvent//mousedown event
        pasteboard: dragPasteboard//pasteboard to pass to receiver
        source: self//object where the image is coming from
        slideBack: YES];//if the drag fails slide the icon back
    [dragImage release];//done with our dragImage
}

// Dragging Source

- (NSDragOperation)draggingSourceOperationMaskForLocal:(BOOL)flag
{
	return NSDragOperationCopy;//send data as copy operation
}

// Dragging Destination

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource]==self || ![self isEditable]) return NSDragOperationNone;
    
    [self setNeedsDisplay: YES];
    return [super draggingEntered: sender]&&(dragging= YES);
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource]==self || ![self isEditable]) return;

    [super draggingExited: sender];
    
    dragging= NO;
    [self setNeedsDisplay: YES];
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
    if([sender draggingSource]==self || ![self isEditable]) return NSDragOperationNone;
    return [super draggingUpdated: sender];
}

- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    return [sender draggingSource]!=self && [super prepareForDragOperation: sender];//[NSImage canInitWithPasteboard: [sender draggingPasteboard]];
} 

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    [super concludeDragOperation: sender];
    
    if([sender draggingSource]==self) return;
    
    dragging= NO;
    [self setNeedsDisplay: YES];
}

@end

/*- (void)pasteboard:(NSPasteboard *)sender provideDataForType:(NSString *)type
{
    //sender has accepted the drag and now we need to send the data for the type we promised
    if([type compare: NSTIFFPboardType]==NSOrderedSame)
            //set data for TIFF type on the pasteboard as requested
        [sender setData: [[self image] TIFFRepresentation] forType: NSTIFFPboardType];
    else if([type compare: NSPDFPboardType]==NSOrderedSame)
        [sender setData: [self dataWithPDFInsideRect: [self bounds]] forType: NSPDFPboardType];
}*/
