//
//  TRMovieInspector(Rollover).m
//  Tryst
//
//  Created by Francisco Tolmasky on Sat Dec 06 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
//

#import "TRGeometry.h"
#import "TRMovieInspector.h"


@implementation TRMovieInspector (Rollover)

// --- Rollovers ---

- (void)updateRolloverImage
{
    NSTextContainer *textContainer= [self textContainer];
    NSLayoutManager *layoutManager= [textContainer layoutManager];
    
    [[self textStorage] attribute: kIdentifierAttributeName atIndex: NSMaxRange(rollover.lhsRange)+1 effectiveRange: &rollover.rhsRange];

    NSRange lhsGlyphRange= [layoutManager glyphRangeForCharacterRange: rollover.lhsRange actualCharacterRange: NULL],
            rhsGlyphRange= [layoutManager glyphRangeForCharacterRange: rollover.rhsRange actualCharacterRange: NULL];
    NSRect  lhsGlyphRect= NSInsetRect([layoutManager boundingRectForGlyphRange: lhsGlyphRange inTextContainer: textContainer],0,0),
            rhsGlyphRect= [layoutManager boundingRectForGlyphRange: rhsGlyphRange inTextContainer: textContainer],
            lastGlyphRect= [layoutManager boundingRectForGlyphRange: NSMakeRange(NSMaxRange(rhsGlyphRange)-1,1) inTextContainer: textContainer];
    
    BOOL requiresAddition= NSMaxX(lastGlyphRect)<NSMaxX(rhsGlyphRect);
    rollover.origin= lhsGlyphRect.origin;
    
    float capRadius= ceilf(NSHeight(lhsGlyphRect)/2.0);
    lhsGlyphRect.origin= NSMakePoint(capRadius,0.0);
    lhsGlyphRect.size.width= ceilf(lhsGlyphRect.size.width);//-2.0;
    NSBezierPath *lhsRollover= [NSBezierPath bezierPathWithRect: lhsGlyphRect];
    [lhsRollover appendBezierPathWithArcWithCenter: NSMakePoint(lhsGlyphRect.origin.x+0.5,NSMinY(lhsGlyphRect)+capRadius) radius: capRadius startAngle: 90.0f endAngle: 270.0f clockwise: NO];
    [lhsRollover closePath];
    
    rhsGlyphRect.size.width= ceilf(rhsGlyphRect.size.width);
    if(rhsGlyphRect.origin.x!=(kRightTabLocation+5.0)) rhsGlyphRect.size.width= ceilf(rhsGlyphRect.size.width)-kRightTabLocation;
    rhsGlyphRect.origin= NSMakePoint([lhsRollover bounds].size.width+1.5,0.0); //3.5-2.0=2.5
    
    if(requiresAddition)
    {
        rhsGlyphRect.size.height-= lastGlyphRect.size.height;
        //lastGlyphRect.size.height+= 3.0;
    }
    if(requiresAddition) rhsGlyphRect.size.width-= capRadius;
    NSBezierPath *rhsRollover= [NSBezierPath bezierPathWithRect: rhsGlyphRect];
    [rhsRollover appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(rhsGlyphRect)+(requiresAddition?2.0:0.0),NSMinY(rhsGlyphRect)+capRadius) radius: capRadius startAngle: 270.0f endAngle: 360.0f];// clockwise: YES];
    [rhsRollover appendBezierPathWithArcWithCenter: NSMakePoint(NSMaxX(rhsGlyphRect)+(requiresAddition?2.0:0.0),NSMaxY(rhsGlyphRect)-capRadius+(requiresAddition?lastGlyphRect.size.height:0)) radius: capRadius startAngle: 0.0f endAngle: 90.0f];// clockwise: YES];
    if(requiresAddition)
        [rhsRollover appendBezierPathWithArcWithCenter: NSMakePoint(NSMinX(rhsGlyphRect)+capRadius,NSMaxY(rhsGlyphRect)-capRadius+lastGlyphRect.size.height) radius: capRadius startAngle: 90.0f endAngle: 180.0f];
    [rhsRollover closePath];
    
    NSSize size= NSMakeSize([lhsRollover bounds].size.width+3.5+[rhsRollover bounds].size.width,rhsGlyphRect.size.height+(requiresAddition?lastGlyphRect.size.height:0.0));
    
    rollover.image= [[NSImage alloc] initWithSize: NSMakeSize(size.width,size.height)];
    
    [rollover.image setCachedSeparately: YES];
    [rollover.image setFlipped: YES];
    
    [rollover.image lockFocus];
    
    [[NSColor lightGrayColor] set];
    [lhsRollover fill];
    [[NSColor blackColor] set];
    [[[NSColor lightGrayColor] colorWithAlphaComponent: 0.5f] set];
    [rhsRollover fill];
    
    [rollover.image unlockFocus];
    
    rollover.origin.x= floorf(rollover.origin.x-capRadius)+2.0;
    rollover.origin.y+= [rollover.image size].height;
}

- (void)drawRolloverIfNecessaryInRect:(NSRect)clipRect
{
    if(rollover.image)
    {
        NSRect rolloverRect= NSMakeRect(rollover.origin.x,rollover.origin.y-[rollover.image size].height,[rollover.image size].width,[rollover.image size].height);
        NSRange selectedRange= [self selectedRange];
        
        // Draw Background
        [[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor clearColor] range: rollover.rhsRange];
        
        [super drawRect: rolloverRect];
        
        // Draw Rollover
        [rollover.image compositeToPoint: rollover.origin operation: NSCompositeSourceOver];
        
        // Draw Text
        [[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor whiteColor] range: rollover.lhsRange];
        [[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor blackColor] range: rollover.rhsRange];
        
        [self setDrawsBackground: NO];
        [self setSelectedRange: NSMakeRange(0,0)];
        [super drawRect: rolloverRect];
        [self setDrawsBackground: YES];
        [self setSelectedRange: selectedRange];
        
        // Restore environment
        [[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor grayColor] range: rollover.lhsRange];
    }
}

- (void)viewWillMoveToWindow:(NSWindow *)newWindow
{
    NSWindow *oldWindow= [self window];
    if (oldWindow)
    {
        NSNotificationCenter *notificationCenter= [NSNotificationCenter defaultCenter];
        [notificationCenter removeObserver: self name: NSWindowDidBecomeKeyNotification object: oldWindow];
        [notificationCenter removeObserver: self name: NSWindowDidResignKeyNotification object: oldWindow];
    }
    if (newWindow)
    {
        // Make sure our window gets mouse moved events so that we can do rollovers.
        [[self window] setAcceptsMouseMovedEvents: YES];
     
		// Register for window becoming and resigning key notifications (The raised editor will draw different if its key, then when it is not key)
        NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
        [notificationCenter addObserver: self selector: @selector(windowKeyStateChangedNotification:) name: NSWindowDidBecomeKeyNotification object: [self window]];
        [notificationCenter addObserver: self selector: @selector(windowKeyStateChangedNotification:) name: NSWindowDidResignKeyNotification object: [self window]];
    }
}

- (void)windowKeyStateChangedNotification:(NSNotification *)aNotification
{
	if([self isEditing]) [self setNeedsDisplay: YES]; //Could be optimizied
}

- (BOOL)becomeFirstResponder
{
	if([self isEditing]) [self setNeedsDisplay: YES]; //Could be optimizied
	return [super becomeFirstResponder];
}

- (BOOL)resignFirstResponder
{
	if([self isEditing]) [self setNeedsDisplay: YES]; //Could be optimizied
	return [super resignFirstResponder];
}

- (void)mouseMoved:(NSEvent *)theEvent
{
    if (![self isEditable]) //Not even necessary any more, handled in drawRect:
    {
        NSTextContainer *textContainer= [self textContainer];
        NSTextStorage *textStorage= [self textStorage];
        NSLayoutManager *layoutManager= [textContainer layoutManager];
        NSPoint localPoint= [self convertPoint: [theEvent locationInWindow] fromView: nil];

        unsigned glyphIndex= [layoutManager glyphIndexForPoint: localPoint inTextContainer: textContainer];
        unsigned characterIndex= [layoutManager characterIndexForGlyphAtIndex: glyphIndex];
        NSRange effectiveRange;
        NSString *label= [textStorage attribute: kLabelAttributeName atIndex: characterIndex effectiveRange: &effectiveRange];

        if (!NSEqualRanges(effectiveRange, rollover.lhsRange))
        {
            if (rollover.image)
            {
                //[[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor blackColor] range: rollover.valueRange];
                NSSize rolloverSize= [rollover.image size];
                rollover.lhsRange= kNotFoundRange;
                rollover.rhsRange= kNotFoundRange;
                [rollover.image release];
                rollover.image= nil;
                [self setNeedsDisplayInRect: NSMakeRect(rollover.origin.x,rollover.origin.y,rolloverSize.width,rolloverSize.height) avoidAdditionalLayout: YES];
            }
            if (label)
            {
                rollover.lhsRange= effectiveRange;
                [self updateRolloverImage];//: currentRolloverCharacterRange];
                //[[self textStorage] addAttribute: NSForegroundColorAttributeName value: [NSColor clearColor] range: rollover.valueRange];
                [self setNeedsDisplayInRect: NSMakeRect(rollover.origin.x,rollover.origin.y,[rollover.image size].width,[rollover.image size].height) avoidAdditionalLayout: YES];
            }
        }
    }
}

@end