//
//  TRMultiValue.h
//  Tryst
//
//  Created by Francisco Tolmasky on Tue Dec 30 2003.
//  Copyright (c) 2004 Francisco Tolmasky. All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.


#import <Foundation/Foundation.h>


@interface TRMultiValue : NSObject <NSCopying, NSMutableCopying, NSCoding>
{
    @protected
    
        NSMutableArray      *identifiers;
        NSMutableArray      *labels;
        NSMutableArray      *values;
}

- (unsigned int)count;
    // Returns the number of value/label pairs

- (id)valueAtIndex:(unsigned int)anIndex;
    // Returns a value at a given index
    // Raises if index is out of bounds

- (NSString *)labelAtIndex:(unsigned int)anIndex;
    // Returns a label at a given index
    // Raises if index is out of bounds

- (NSString *)identifierAtIndex:(unsigned int)anIndex;
    // Returns a identifier at a given index
    // Raises if index is out of bounds
    
- (unsigned int)indexForIdentifier:(NSString *)anIdentifier;
    // Returns the index of a given identifier
    // Returns NSNotFound if not found

@end

@interface TRMutableMultiValue : TRMultiValue

- (NSString *)addValue:(id)value withLabel:(NSString *)aLabel;
    // Adds a value with its label
    // Returns the identifier if successful. nil otherwise
    // Raises if value or label are nil
    // Note: No type checking is made when adding a value. But trying to set a multivalue property
    // with a multivalue that doesn't have all its values of the same type will return an error

- (NSString *)insertValue:(id)value withLabel:(NSString *)aLabel atIndex:(unsigned int)anIndex;
    // Insert a value/label pair at a given index
    // Returns the identifier if successful. nil otherwise
    // Raises if value or label are nil or the index is out of bounds
    // Note: No type checking is made when adding a value. But trying to set a multivalue property
    // with a multivalue that doesn't have all its values of the same type will return an error

- (BOOL)removeValueAndLabelAtIndex:(unsigned int)anIndex;
    // Removes a value/label pari at a given index
    // Raises if the index is out of bounds

- (BOOL)replaceValueAtIndex:(unsigned int)anIndex withValue:(id)aValue;
    // Replaces a value at a given index
    // Raises if the index is out of bounds or the value is nil

- (BOOL)replaceLabelAtIndex:(unsigned int)anIndex withLabel:(NSString*)aLabel;
    // Replaces a label at a given index
    // Raises if the index is out of bounds or the label is nil

@end